/* Header for module lyrics, generated by p2c 1.21alpha-07.Dec.93 */
#ifndef LYRICS_H
#define LYRICS_H


/* Everything related to lyrics */

#ifndef GLOBALS_H
#include "globals.h"
#endif


#ifdef LYRICS_G
# define vextern
#else
# define vextern extern
#endif

extern void initLyrics(void);
extern void clearTags(void);
extern void reviseLyrics(void);
extern void lyrTranslate(Char *P, boolean *numbered);
extern void getSyllable(voice_index voice, Char *pretex);
extern Char *lyricsReport(Char *Result, voice_index voice);
extern void maybeLyrics(voice_index voice, paragraph_index parline, Char *w);
extern void extractLyrtag(voice_index voice, Char *note);
extern void lyricsAdjust(voice_index voice, Char *note);
extern void assignLyrics(stave_index stave, Char *lyrassign);
extern void lyricsParagraph(void);
extern boolean hasVerseNumber(voice_index voice);


#undef vextern

#endif /*LYRICS_H*/

/* End. */
